#####################################
#
# for any questions regarding the following script, please contact corresponding author Cline Albert
# celine.albert21@gmail.com
#
#####################################

rm(list=ls())

#####################################
#
# Method adapted from https://www.r-bloggers.com/2018/03/advanced-techniques-with-raster-data-part-3-exercises/
#
#####################################


# Workspace
  require(splitstackshape); require(dplyr); library(purrr)
  
  library(visreg); library(statmod); library(gamm4); library(lme4); library(mgcv); library(gratia);library(lme4); library(lattice)
library(dotwhisker);library(broom); library(magicfor); library(sp) ;library (raster); library(gstat); library(automap);library(RColorBrewer)
library(tidyr);library(dplyr);library(ggplot2);library(broom); library(rworldmap);library(sf); library(rgdal)

#####################################
#
# Prepare data
#
#####################################

# Load & format dataset
  data <- read.csv("/Users/calbert/Desktop/IRMA/mergedIRMA.csv", sep=";", dec=",")
  data <- read.csv("/Users/celin/Desktop/IRMA/mergedIRMA.csv", sep=";", dec=",")
  
  data %>% 
    dplyr::mutate(across(.cols = c(Species, Ring_number, YEAR, Sampling_site), .fns = as.factor))  %>% 
    dplyr::filter(!is.na(Hg_HF_BF_Winter_Spatial) & !is.na(lon_medoids) & lat_medoids >= 40.0) %>%
    dplyr::distinct(Ring_number, YEAR, lon_medoids, lat_medoids, .keep_all = TRUE) ->
  data


# create quick function to stratify (for use in loop)
  # I removed the Year bc no variations in Hg and we decided to pool years
  stratsample <- function(x){
    x %>%
      splitstackshape::stratified(., group =  c("Species", "Sampling_site", "Ring_number"), size = 1) %>%
      dplyr::arrange(Species, Sampling_site, Ring_number)
  }
  
  
# create a list of N sub-datasets (each one randomly sampled)
  N = 10
  df <- purrr::map(1:N, ~stratsample(data))
  
# run models and store each output as element of a list (might not be the best option when N is large)
  lmm.out <- purrr::map(df, ~ lmerTest::lmer(log(Hg_HF_BF_Winter_Spatial)~(lon_medoids)+ (lat_medoids)+ 
                                            (1|Sampling_site:Species),data=.x, REML=T)) #Here, species is nested inside sampling sites
  

  # check outputs
  i = 10
  summary(lmm.out[[i]])   
  anova(lmm.out[[i]])

  plot(lmm.out[[i]])
  
  qqnorm(resid(lmm.out[[i]]))
  qqline(resid(lmm.out[[i]]))
  plot(lmm.out[[i]], sqrt(abs(resid(.))) ~ fitted(.), type = c("p", "smooth"))
  qqmath(lmm.out[[i]], id = 0.05)
 
  plot(fixef(lmm.out[[i]]))#plot fixe effects
  
# library("ggpubr") 
# library(MASS)
# ggdensity(data$Hg_HF_BF_Winter_Spatial)
# ggqqplot(data$Hg_HF_BF_Winter_Spatial)
  

  #hist(pvalue_lon)
  #hist(pvalue_lat)
  #hist(slope_lon)
  #hist(slope_lat)


# Loop for predictions

Predictions_Outputs<-purrr::map(df,~predict(lmm.out[[i]], newdata=.x))

All<-map2(df, Predictions_Outputs, ~cbind(.x, predictions=.y))

#plot prediction, just change [[i]] to get the map you want
ggplot() +
  geom_point(data = All[[1]], aes(x = lon_medoids, y = lat_medoids, color=predictions), size = 4, 
             shape = 16)+ scale_colour_gradientn(colours=terrain.colors(18))

summary(All[[7]])

#################################################
#
# Create a grid where to plot the data
#
#################################################


proj4Str<-"+proj=longlat +datum=WGS84 +no_defs +ellps=WGS84 +towgs84=0,0,0"
#statPoints<-purrr::map(All,~SpatialPointsDataFrame(coords = .x[[i]][, c("lon_medoids", "lat_medoids")], 
 #                                  data = .x, 
  #                                 proj4string = CRS(proj4Str)))

# have the datatable into a spatialPointDataFrame
statPoints<-list()
for (i in 1:10) {
  statPoints[[i]]<-SpatialPointsDataFrame(coords = All[[i]][, c("lon_medoids", "lat_medoids")], 
                                     data = All[[i]], 
                                     proj4string = CRS(proj4Str))
}

summary(statPoints[[2]])
plot(statPoints[[1]])

#Create a raster that follow the extent of our dataset to create our grid
#there might be a better/easier way to do it, but at least it works
rast <- raster(ncol = seq(from=-85.74697, to=68.6525, by=1), nrow = seq(from=40.01304, to=79.24311, by=1))
rast
#proj4string(rast)
#proj4string(rast)<-CRS("+proj=longlat +datum=WGS84 +no_defs +ellps=WGS84 +towgs84=0,0,0")
data1<-SpatialPointsDataFrame(coords = data[, c("lon_medoids", "lat_medoids")], 
                                         data = data, 
                                         proj4string = CRS(proj4Str))

extent(data1)

extent(rast) <- extent(data1)
extent(rast)
rast
s <- raster(nrow = 40, ncol = 155)
extent(s) <- extent(rast)
rast <- resample(rast, s, method = 'bilinear') # resample output
res(rast)
#Now we have a raster with cells 1x1


# I can now raster each variable of my dataset
X<-purrr::map(statPoints, ~rasterize(.x, rast, .x$X))
YEAR<-purrr::map(statPoints, ~rasterize(.x, rast, as.numeric(.x$YEAR)))
Species<-purrr::map(statPoints, ~rasterize(.x, rast, as.numeric(.x$Species)))
Sampling_site<-purrr::map(statPoints, ~rasterize(.x, rast, as.numeric(.x$Sampling_site)))
Ring_number<-purrr::map(statPoints, ~rasterize(.x, rast, as.numeric(.x$Ring_number)))
Hg_HF_BF_Winter_Spatial<-purrr::map(statPoints, ~rasterize(.x, rast, .x$Hg_HF_BF_Winter_Spatial))
lon_medoids<-purrr::map(statPoints, ~rasterize(.x, rast, .x$lon_medoids))
lat_medoids<-purrr::map(statPoints, ~rasterize(.x, rast, .x$lat_medoids))
predictions<-purrr::map(statPoints, ~rasterize(.x, rast, .x$predictions))

# And then stack them
for (i in 1:10) {
  stack(X[[i]])
  stack(YEAR[[i]])
  stack(Species[[i]])
  stack(Sampling_site[[i]])
  stack(Ring_number[[i]])
  stack(Hg_HF_BF_Winter_Spatial[[i]])
  stack(lon_medoids[[i]])
  stack(lat_medoids[[i]])
  stack(predictions[[i]])
}

# test : plotting the different predictions
plot(predictions[[10]])
plot(predictions[[1]])

# we create a new Large list where all variables will be stacked (so as rasterStack)
A<-stack()
A<-list()
for (i in 1:10) {
  A[[i]]<-stack(X[[i]], YEAR[[i]], Species[[i]], Sampling_site[[i]], Ring_number[[i]], 
                   Hg_HF_BF_Winter_Spatial[[i]], lon_medoids[[i]],lat_medoids[[i]], predictions[[i]])
}

# When plotting the result, we get each layer
plot(A[[10]])

#extract the names of each layer
names(A[[10]])

# rename them following our dataset
for (i in 1:10) {
  names(A[[i]]) <- c("X", "YEAR", "Species", "Sampling_site", "Ring_number", 
                     "Hg_HF_BF_Winter_Spatial", "lon_medoids","lat_medoids", "predictions")
}

proj4string(A[[10]])

plot(A[[10]])

names(A[[10]])

plot(A[[1]])

#################################################################
#
# Run the kriging
#
#################################################################

# Create a temporary SpatialPointsDF object to store LMM residuals

statPointsTMP <- statPoints
crs(statPointsTMP[[i]]) <- crs(proj4Str)
rstPixDF<-list()
for (i in 1:10) {
  rstPixDF[[i]] <- as(A[[i]][[6]], "SpatialPixelsDataFrame")
}
#[[6]] = Hg concentrations 

crs(statPointsTMP[[i]]) <- crs(rstPixDF[[i]])
proj4string(rstPixDF[[1]])

# store the residuals into the dataset
for (i in 1:10) {
  statPointsTMP[[i]]@data<- cbind(statPointsTMP[[i]]@data, residLMM = resid(lmm.out[[i]]))
}


# Define the kriging parameters and fit the variogram using OLS
formMod<-list()
for (i in 1:10) {  
formMod[[i]] <- residLMM~ 1
}

mod<-list() # store the parameters for the variogram
variog<-list() # store the results of the variogram
variogFitOLS<-list() # store the fitting of the variogram

for (i in 1:10) {  
  mod[[i]] <- vgm(model  = "Sph" )
  variog[[i]] <- variogram(formMod[[i]], statPointsTMP[[i]])
  variogFitOLS[[i]] <- fit.variogram(variog[[i]], model = mod[[i]])
}

# Plot the results
plot(variog[[10]], variogFitOLS[[10]], main="Semi-variogram of LMM residuals")
plot(variog[[1]], variogFitOLS[[1]], main="Semi-variogram of LMM residuals")

# store the results of the krigind
residKrigMap<-list()
for (i in 1:10) { 
residKrigMap[[i]] <- krige(formula = formMod[[i]] ,
                      locations = statPointsTMP[[i]], 
                      model = variogFitOLS[[i]],
                      newdata = rstPixDF[[i]])
}


# residKrigMap is a "SpatialPixelDataFrame". 
# the other data are Raster Layer
# So I need to change residKrigMap into a raster layer

residKrigMap_pred<-list() # Store the predictions of the kriging
residKrigMap_var<-list()  # store the variance of the krigins

for (i in 1:10) {
  residKrigMap_pred[[i]] <- rasterize(x=statPoints[[i]][, c("lon_medoids", "lat_medoids")], # lon-lat data
                                 rast, # raster object
                                 field=residKrigMap[[i]]$var1.pred) # vals to fill raster with
  
  residKrigMap_var[[i]] <- rasterize(x=statPoints[[i]][, c("lon_medoids", "lat_medoids")], # lon-lat data
                                     rast, # raster object
                                     field=residKrigMap[[i]]$var1.var) # vals to fill raster with
  
                            
}

par(mfrow=c(1,1))

nb.cols<-18
pal2 <- colorRampPalette(brewer.pal(8,"PuOr"))(nb.cols)
plot(residKrigMap_pred[[10]],main="[Hg]\n(LMM regression-kriging predictions)",col=pal2,
     xlab="Longitude", ylab="Latitude", cex.main=0.8, cex.axis=0.7, cex=0.8)
plot(residKrigMap_var[[1]],main="[Hg]\n(GAM regression-kriging variance)",col=rev(pal2),
     xlab="Longitude", ylab="Latitude", cex.main=0.8, cex.axis=0.7, cex=0.8)

# create a new list of raster stack to add the new data from the kriging to the original dataset
B<-stack()
B<-list()
for (i in 1:10) {
  B[[i]]<-stack(X[[i]], YEAR[[i]], Species[[i]], Sampling_site[[i]], Ring_number[[i]], 
                Hg_HF_BF_Winter_Spatial[[i]], lon_medoids[[i]],lat_medoids[[i]], predictions[[i]], 
                residKrigMap_pred[[i]], residKrigMap_var[[i]])
  names(B[[i]]) <- c("X", "YEAR", "Species", "Sampling_site", "Ring_number", 
                     "Hg_HF_BF_Winter_Spatial", "lon_medoids","lat_medoids", "predictions", "pred_krig", "se_krig")
  #change name layers
}

plot(B[[1]])



# change to raster layer to have the same format
# residKrigMap_Pred <- as(residKrigMap$var1.pred, "RasterLayer")
# We can now add the predictions of the LMM to the predictions of the kriging
for (i in 1:10) {
  residKrigMap[[i]]<-B[[i]]$pred_krig+B[[i]]$predictions
  stack(residKrigMap[[i]])
}
plot(residKrigMap[[10]])

#residKrigMap_se<-B$se_krig+B$s
#stack(residKrigMap_se)
# create a new list of raster stack to add the new data  "residKrigMap" to the previous list "B"
C<-stack()
C<-list()
for (i in 1:10) {
  C[[i]]<-stack(X[[i]], YEAR[[i]], Species[[i]], Sampling_site[[i]], Ring_number[[i]], 
                Hg_HF_BF_Winter_Spatial[[i]], lon_medoids[[i]],lat_medoids[[i]], predictions[[i]], 
                residKrigMap_pred[[i]], residKrigMap_var[[i]],residKrigMap[[i]])
  names(C[[i]]) <- c("X", "YEAR", "Species", "Sampling_site", "Ring_number", 
                     "Hg_HF_BF_Winter_Spatial", "lon_medoids","lat_medoids", "predictions", "pred_krig", "se_krig", "residKrigMap")
  #change name layers
}

plot(C[[10]]$residKrigMap)
plot(C[[1]]$residKrigMap)
plot(C[[5]]$residKrigMap)

#On peut repasser en spatialPixelDataFrame pour pouvoir utiliser spplot. 
#residKrigMap <- as(residKrigMap, "SpatialPixelsDataFrame")
#rstPredGAM_2 <- as(rstPredGAM, "SpatialPixelsDataFrame")
#gamKrigMap <- rstPredGAM_2+ residKrigMap



### Calculate a SD list depending on the number of iteration
C_bis <-list()
for (i in 1:10) {
 C_bis[[i]]<- raster::subset(C[[i]], c("residKrigMap"))
}

C_bis1<-stack(C_bis)

#create a list to store the results of each sd
sd_essay<-list()

mean_narm = function(x,...){mean(x,na.rm=TRUE)}
sd_narm = function(x,...){sd(x,na.rm=TRUE)}

# test to be sure that the results from the loop are in accordance with the result for each overlay

# sd_essay0<-overlay(C_bis1[[1:1]], fun = sd_narm )
# sd_essay1<-overlay(C_bis1[[1:2]], fun = sd_narm )
# sd_essay2<-overlay(C_bis1[[1:3]], fun = sd_narm )
# sd_essay3<-overlay(C_bis1[[1:4]], fun = sd_narm )
# sd_essay4<-overlay(C_bis1[[1:5]], fun = sd_narm )
# sd_essay5<-overlay(C_bis1[[1:6]], fun = sd_narm )

for (i in 1:9) {
  sd_essay[[i]]<-calc(C_bis1[[1:(i+1)]], fun = sd_narm)
}

sd_essay[[3]]

plot(sd_essay[[1]])
plot(sd_essay[[3]])

# need to stack the sd_essay to calculate mean sd depending on the number or iterations
sd_essayV2<-stack(sd_essay)
mean_sd_essay <- cellStats(sd_essayV2, "mean") #calculate mean sd for all the rasters, not per cell
plot(mean_sd_essay)

######### Have a mean value from all layers #########
# Calculate mean

D <- do.call(overlay, c(C, fun =mean_narm ))
plot(D$layer.12)

D1 <- do.call(overlay, c(C, fun = sd_narm)) 
plot(D1$layer.12)


### Whitening 

D <- as(D, "SpatialGridDataFrame")
D <- as(D, "SpatialPixelsDataFrame")

D1 <- as(D1, "SpatialGridDataFrame")
D1 <- as(D1, "SpatialPixelsDataFrame")

# Plot the predictions/variance:

CropWorld3<-readOGR(dsn="/Users/celin/Documents/LIENSs/Thse/Chapitre 3/Analyses/AllSPecies/LIAK",layer="decoup1")
CropWorld3<-readOGR(dsn="/Users/calbert/Desktop/IRMA/decoup1",layer="decoup1")
#CropWorld3<-readOGR(dsn="/Users/Nuninger Lise/Desktop/IRMA/decoup1",layer="decoup1")
#z.plot <- spplot(C["z"], col.regions=bpy.colors(), scales=list(draw=TRUE), sp.layout=list("sp.points", pch="+", col="black", LIAK.Subset5))
z.plot <- spplot(D["layer.12"], sp.layout=list(CropWorld3, fill="grey", first=F),
                 col.regions=rev(pal2), scales=list(draw=TRUE), ylim=c(40,80),xlim=c(-85,69),
                 main=list(cex=0.8, label="[Hg]  All species\n(LMM regression-kriging): predictions IRMA"))
z.plot
e.plot <- spplot(D1["layer.12"], sp.layout=list(CropWorld3, fill="snow1", first=F),
                 col.regions=rev(pal2), scales=list(draw=TRUE), 
                 main=list(cex=0.8, label="[Hg] All species\n(GAM regression-kriging): variance IRMA"))
e.plot

print(z.plot)
print(e.plot)


# Map only Hg 
nb.cols<-18
pal2<-colorRampPalette(brewer.pal(8, "PuOr"))(nb.cols) #color blind friendly

# pal2<-colorRampPalette(brewer.pal(8, "RdYlGn"))(nb.cols) # red green

# map with colors depending on [Hg]
display.brewer.pal(n = 11, name = 'PuOr')
brewer.pal(n = 11, name = "PuOr")
z.plot_Hg <- spplot(data1["Hg_HF_BF_Winter_Spatial"], cex=0.4, cuts = c(0,2,5,15),col.regions=c("#8073AC","#FDB863","#B35806"),
                    sp.layout=list(CropWorld3, fill="grey", first=F),
                scales=list(draw=TRUE), ylim=c(40,80),xlim=c(-85,69),
                 main=list(cex=0.8, label="[Hg]  All species\n(LMM regression-kriging): predictions IRMA"))
z.plot_Hg

# map without colors

z.plot_Hg <- spplot(data1["Hg_HF_BF_Winter_Spatial"], cex=0.4, pch = 16, col.regions=("black"),
                    sp.layout=list(CropWorld3, fill="grey", first=F),
                    scales=list(draw=TRUE), ylim=c(40,80),xlim=c(-85,69),
                    main=list(cex=0.8, label="[Hg]  All species\n(LMM regression-kriging): predictions IRMA"))
z.plot_Hg

# Save data as an R object

saveRDS(D, file="D.rds")

################################################
#
#You can also plot with the following script when you have the rds file : 
#
################################################

state<-readOGR(dsn="/Users/celin/Desktop/IRMA/decoup1",layer="decoup1")

nb.cols<-18
library(RColorBrewer)
pal2 <- colorRampPalette(brewer.pal(8,"RdYlGn"))(nb.cols)
library(RStoolbox)
library(sp)

mean_lmmok<-readRDS("/Users/celin/Desktop/IRMA/Regression Kriging/Kriging on residuals", file = "mean_lmmok.rds")
sd_lmmok<-readRDS("/Users/celin/Desktop/IRMA/Regression Kriging/Kriging on residuals", file = "sd_lmmok.rds")


mean_lmmok1<-ggR(mean_lmmok, geom_raster = TRUE) +
  scale_fill_gradientn("", colours = rev(pal2))+
  theme_bw()+
  theme(axis.title.x=element_blank(),
        axis.text.x=element_blank(),
        axis.ticks.x=element_blank(),
        axis.title.y=element_blank(),
        axis.text.y=element_blank(),
        axis.ticks.y=element_blank())+
  ggtitle("Predicted values of LMM and OK")+
  theme(plot.title = element_text(hjust = 0.5))+
  geom_polygon(data = state, aes( x = long, y = lat, group = group), fill="grey", color="white")
mean_lmmok1

sd_lmmok1<-ggR(sd_lmmok, geom_raster = TRUE) +
  scale_fill_gradientn("", colours = rev(pal2))+
  theme_bw()+
  theme(axis.title.x=element_blank(),
        axis.text.x=element_blank(),
        axis.ticks.x=element_blank(),
        axis.title.y=element_blank(),
        axis.text.y=element_blank(),
        axis.ticks.y=element_blank())+
  ggtitle("")+
  theme(plot.title = element_text(hjust = 0.5))+
  geom_polygon(data = state, aes( x = long, y = lat, group = group), fill="grey", color="white")
sd_lmmok1